// C++ Vererbung: Geom. Objekte
// Abstrakte Klasse Bigobj
//
// Klaus Kusche, 2011

#ifndef _BIGOBJ_HXX
#define _BIGOBJ_HXX

#include "color.hxx"

class BigObj {
  public:
    BigObj(const Color &color, int x, int y, int w, int h) :
      mRGB(color), mX(x), mY(y), mW(w), mH(h), mZ(0) {}  // draw in abgel. Klasse
    BigObj(const BigObj &orig) :
      mRGB(orig.mRGB), mX(orig.mX), mY(orig.mY),
      mW(orig.mW), mH(orig.mH), mZ(0) {
        // nicht einfach kopieren, sondern hndisch setzen,
        // damit die Kopie frisch in MzBuf eingefgt wird
        setZ(orig.mZ);
      }  // draw in abgel. Klasse
    virtual ~BigObj() {setZ(0);}  // 1. virtuell, 2. undraw nur in abgel. Klasse!

    Color getColor() const {return mRGB;}
    void setColor(const Color &color) {
      mRGB = color; draw();}
    
    int getX() const {return mX;}
    int getY() const {return mY;}
    int getW() const {return mW;}
    int getH() const {return mH;}
    int getZ() const {return mZ;}

    virtual void setPos(int x, int y);
    virtual void setSize(int w, int h);
    void setZ(int z);
    virtual void move(int xOffset, int yOffset);
    virtual void scale(int percentX, int percentY);
    virtual void rotate();

    virtual void draw() = 0;   // pure virtual
    virtual void undraw() = 0; // pure virtual

    virtual BigObj *clone() const = 0;

    static void drawAll();

  protected:  // fr abgel. Klassen zugreifbar
    Color mRGB;
    int mX, mY;    // Position des Mittelpunktes
    int mW, mH;    // Width und Height (jeweils ab Mittelpunkt)

  private:    // nur fr diese Klasse
    int mZ;        // Z-Reihenfolge zum Neuzeichnen
    static const int MzBufSize = 256;
    static BigObj *MzBuf[MzBufSize];
    static int MzBufCnt;

  private:
    BigObj &operator=(const BigObj &val); // wegdefinieren (bleibt undef)!
};

#endif
